# Copyright (c) 2023 TheraPanacea. All rights Reserved.
# Unauthorized copying of this file, via any medium is strictly prohibited
# Proprietary and confidential
"""Pushes models and operators to the OpStore."""
import http
import logging
import os

import requests
import yaml

from artist import client as client_lib

SERVICE = 'ops_store'


def push_models(
    client: client_lib.ARTClient,
    model_paths: dict[str, str],
    minio_host: str | None = None,
) -> dict[str, str]:
    """Pushes models from local disk to ART platform."""
    result = {}
    root = client.service(SERVICE)
    for name, path in model_paths.items():
        # Asks for an upload.
        payload = {'name': name}
        r = client.post(root, 'models', json=payload)
        if r.status_code != http.client.OK:
            logging.error('Cannot request upload')
            continue
        resp = r.json()
        uid = resp.get('uid')
        url = resp.get('url')
        if uid is None or url is None:
            logging.error('Insufficient info in response: %s', resp)
            continue

        # Performs the upload.
        with open(path, 'rb') as fp:
            headers = {'Content-type': 'application/octet-stream'}
            if minio_host is not None:
                headers['HOST'] = minio_host
            r = requests.put(url, data=fp, headers=headers, timeout=600)
        if r.status_code != http.client.OK:
            logging.error('Error %s uploading files.', r.status_code)

        # Notifies upload completion.
        r = client.post(root, 'models/complete', json={'uid': uid})
        if r.status_code != http.client.OK:
            logging.error('Cannot notify upload completion of %s.', uid)
            continue
        resp = r.json()
        result[name] = resp.get('uid')

    return result


def add_operator(
    client: client_lib.ARTClient,
    name: str,
    target: str,
    version: str = 'latest',
    *,
    params: dict | None = None,
    models: dict[str, str] | None = None,
    keep_unspecified_models: bool = False,
    family: str = '',
    description: str = '',
    processor: str = 'k8s',
    bump: str = 'minor',
    min_gpus: int = 0,
    max_gpus: int = 0,
    ram: int | None = None,
    gpu_ram: int | None = None,
    force: bool = False,
    minio_host: str | None = None,
):
    """Adds an operator with the given specifications."""
    if models is not None:
        to_push = {k: v for k, v in models.items() if os.path.exists(v)}
        models.update(push_models(client, to_push, minio_host))
    payload = {
        'name': name,
        'target': target,
        'processor': processor,
        'params': {} if params is None else params,
        'models': {} if models is None else models,
        'version': version,
        'details': {
            'family': family,
            'description': description,
        },
        'requirements':
            {
                'min_gpus': min_gpus,
                'max_gpus': max_gpus,
                'ram': ram,
                'gpu_ram': gpu_ram,
            },
        'keep_unspecified_models': keep_unspecified_models,
        'bump': bump,
    }
    root = client.service(SERVICE)
    resp = client.post(root, f'/operators?force={int(force)}', json=payload)
    if resp.status_code != http.client.OK:
        logging.error('Cannot update %s:%s', name, version)
        return
    resp = resp.json()
    return resp.get('name'), resp.get('version')


def add_operator_from_yaml(
    client: client_lib.ARTClient,
    filename: str,
    force: bool = False,
):
    """Reads a yaml file and add the operator."""
    with open(filename, 'r', encoding='utf8') as fp:
        data = yaml.safe_load(fp)
    return add_operator(client=client, force=force, **data)


def catalog(client: client_lib.ARTClient):
    """Returns the whole OpsStore catalog."""
    root = client.service(SERVICE)
    resp = client.get(root, 'operators')
    if resp.status_code != http.client.OK:
        logging.error('Cannot fetch operators')
        return
    return resp
