# Copyright (c) 2023 TheraPanacea. All rights Reserved.
# Unauthorized copying of this file, via any medium is strictly prohibited
# Proprietary and confidential
"""Get image and ROI from Viewer"""
import base64
import json
import logging

from artist import client as client_lib

SERVICE = 'viewer'


def get_slice(
    client: client_lib.ARTClient, slice_idx: int, volume_uid: str
) -> bytes | None:
    """Get a slice from given volume."""
    root = client.service(SERVICE)

    resp = client.get(root, f'viewer/volume/{volume_uid}/slice/{slice_idx}')
    if resp.status_code != 200:
        logging.error(
            'Error %i getting slicer: %s', resp.status_code, resp.content
        )
        return None

    resp = resp.json()
    return {
        'image': base64.b64decode(resp['data']),
        'shape': resp.get('shape'),
        'dtype': resp.get('dtype')
    }


def add_roi(
    client: client_lib.ARTClient,
    name: str,
    volume_uid: str,
    color: int = None,
    label: str = None
) -> str | None:
    """Add a roi to the default roi set of volume
    Args:
        client : artist client
        name: str, internal name of the ROI
        volume_uid; str, UID of target volume
        color: int, ROI prefered color (0xRRGGBB)
        label: str, Human readable name

    Returns:
        str, UID of created ROI, None if error
    """
    payload = {
        'name': name,
        'color': color,
        'label': label,
        'volume_uid': volume_uid
    }
    root = client.service(SERVICE)
    resp = client.post(root, 'roi/create', json=payload)
    if resp.status_code != 200:
        logging.error(
            'Error getting %i roi set: %s', resp.status_code, resp.content
        )
        return None
    resp = resp.json()
    return resp.get('uid')


def get_roi_set(
    client: client_lib.ARTClient,
    volume_uid: str,
    name: str = None
) -> str | None:
    """Get a roi set for a given volume

    Args:
        client : artist client
        volume_uid: str, uid of target volume
        name: str|None, target roi set, none for default one
    """
    root = client.service(SERVICE)
    resp = client.get(root, f'roi_set/image/{volume_uid}')
    if resp.status_code != 200:
        logging.error(
            'Error %i getting roi set: %s', resp.status_code, resp.content
        )
        return None

    resp = resp.json()

    logging.debug('Found %i Roi set(s) for volume %s', len(resp), volume_uid)

    #trick to get default one
    if name is None:
        name = volume_uid

    result = None
    for roi_set in resp:
        roi_set_uid = roi_set.get('uid')
        roi_set_name = roi_set.get('name')
        if roi_set_name == name:
            result = roi_set_uid
        logging.debug(
            '\t - Roi set %s found with uid=%s', roi_set_name, roi_set_uid
        )

    #return requested one
    return result


def get_rois(client: client_lib.ARTClient, roi_set_uid: str) -> list[str]:
    """ Get all ROIs from a given ROI set

    Args:
        client : artist client
        roi_set_uid: str, uid of target roi set

    Returns:
        List of ROI uids
    """
    root = client.service(SERVICE)
    resp = client.get(root, f'roi_set/{roi_set_uid}/rois')
    if resp.status_code != 200:
        logging.error(
            'Error %i getting rois from roi set: %s', resp.status_code,
            resp.content
        )
        return []

    resp = resp.json()
    logging.debug('Found %i Roi(s) in Roi Set %s', len(resp), roi_set_uid)
    return resp


def get_roi(client: client_lib.ARTClient, roi_uid: str):
    root = client.service(SERVICE)
    resp = client.get(root, f'roi/{roi_uid}')
    if resp.status_code != 200:
        logging.error(
            'Error %i getting roi %s: %s:',
            resp.status_code,
            roi_uid,
            resp.content,
        )
        return None

    return resp.json()


def set_contours(
    client: client_lib.ARTClient,
    roi_uid: str,
    contours: list[dict],
    coordinates_system='VOXEL'
) -> list[dict] | None:
    """ set contours to a ROIs from a given ROI set
    Args:
        client : artist client
        roi_set_uid: str, uid of target roi set
        contours: list[contour] with contour being a dict
        {'points':list[point] } and point being list of [float,float float].
        ex : [{'points':[[0,0,0],[10,0,0],[10,10,0],[0,10,0]]}]
        coordinates_system : str, can be VOXEL (row,col,slice) or
            dicom (Righ/Left,Anterior/Posterior,Feet/Head)
    Returns:
        Contours recorded
    """
    payload = {
        'contours': {
            'data': contours,
            'id': roi_uid
        },
        'coordinates_system': coordinates_system
    }

    root = client.service(SERVICE)
    resp = client.post(root, f'roi/{roi_uid}/contours', json=payload)
    if resp.status_code != 200:
        logging.error(
            'Error %i setting contours: %s', resp.status_code, resp.content
        )
        return None

    resp = resp.json()
    #retrieve contour set as resp
    return resp['contours'].get('data') if resp.get('contours') else None


def get_contours(
    client: client_lib.ARTClient,
    roi_uid: str,
    volume_uid: str | None = None,
    coordinates_system='VOXEL'
) -> list[dict]:
    """ Get contours of a ROI for a given volume, default if None

    Args:
        client : artist client
        roi_set_uid: str, uid of target roi set
        coordinates_system : str, can be VOXEL (row,col,slice) or
            dicom (Righ/Left,Anterior/Posterior,Feet/Head)
        volume_uid : str, get contours from this volume, default if None
    Returns:
        Contours
    """
    payload = {
        'coordinates_system': coordinates_system,
        'volume_uid': volume_uid
    }

    root = client.service(SERVICE)
    resp = client.get(root, f'roi/{roi_uid}/contours', params=payload)
    if resp.status_code != 200:
        logging.error(
            'Error %i getting contours: %s', resp.status_code, resp.content
        )
        return None

    resp = resp.json()
    return resp['contours'].get('data') if resp.get('contours') else None


def threshold(
    client: client_lib.ARTClient,
    value: float,
    volume_uid: str,
    operator: str = '>',
):
    """Thresholds a volume and make a ROI out of it."""
    payload = {
        'tool': 'threshold',
        'volume': volume_uid,
        'params': json.dumps({
            'comparison': operator,
            'value': value
        })
    }
    root = client.service(SERVICE)
    resp = client.post(root, 'edition_tools/launch', json=payload)
    if resp.status_code != 200:
        logging.error(
            'Error %i in %s : %s', resp.status_code, payload['tool'],
            resp.content
        )
        return None

    resp = resp.json()
    logging.debug('Tool %s done', payload['tool'])
    return resp


def connected_components(
    client: client_lib.ARTClient, roi_uid: str, connectivity: int = 26
):
    """Compute connected components rois out of a single one."""
    payload = {
        'tool': 'cc',
        'roi': roi_uid,
        'params': json.dumps({'connectivity': connectivity})
    }
    root = client.service(SERVICE)
    resp = client.post(root, 'edition_tools/launch', json=payload)
    if resp.status_code != 200:
        logging.error(
            'Error %i in %s : %s', resp.status_code, payload['tool'],
            resp.content
        )
        return None

    resp = resp.json()
    logging.debug('Tool %s done', payload['tool'])
    return resp


def dilate(
    client: client_lib.ARTClient,
    rois: list[str],
    size: float = 1,
    unit: str = 'mm',
    overwrite: bool = False,
):
    """Dilates a given list of rois with a given margin."""
    params = json.dumps({
        'size': size,
        'unit': unit,
        'overwrite': overwrite,
    })
    payload = {'tool': 'dilation', 'rois': rois, 'params': params}
    root = client.service(SERVICE)
    resp = client.post(root, 'edition_tools/launch', json=payload)
    if resp.status_code != 200:
        logging.error(
            'Error %i in %s : %s', resp.status_code, payload['tool'],
            resp.content
        )
        return None

    resp = resp.json()
    logging.debug('Tool %s done', payload['tool'])
    return resp


def erode(
    client: client_lib.ARTClient,
    rois: list[str],
    size: float = 1,
    unit: str = 'mm',
    overwrite: bool = False,
):
    """Dilates a given list of rois with a given margin."""
    params = json.dumps({
        'size': size,
        'unit': unit,
        'overwrite': overwrite,
    })
    payload = {'tool': 'erosion', 'rois': rois, 'params': params}
    root = client.service(SERVICE)
    resp = client.post(root, 'edition_tools/launch', json=payload)
    if resp.status_code != 200:
        logging.error(
            'Error %i in %s : %s', resp.status_code, payload['tool'],
            resp.content
        )
        return None

    resp = resp.json()
    logging.debug('Tool %s done', payload['tool'])
    return resp


def fill_holes(
    client: client_lib.ARTClient,
    rois: list[str],
    overwrite: bool = False,
):
    """Dilates a given list of rois with a given margin."""
    params = json.dumps({'overwrite': overwrite})
    payload = {'tool': 'fill_holes', 'rois': rois, 'params': params}
    root = client.service(SERVICE)
    resp = client.post(root, 'edition_tools/launch', json=payload)
    if resp.status_code != 200:
        logging.error(
            'Error %i in %s : %s', resp.status_code, payload['tool'],
            resp.content
        )
        return None

    resp = resp.json()
    logging.debug('Tool %s done', payload['tool'])
    return resp


def smart_scissors(
    client: client_lib.ARTClient,
    volume: str,
    roi: str,
    line: list[list[float]],
    coordinates_system: str = 'WORLD',
):
    """Dilates a given list of rois with a given margin."""
    params = json.dumps({'coordinates_system': coordinates_system})
    payload = {
        'tool': 'scissors',
        'roi': roi,
        'volume': volume,
        'line': line,
        'params': params
    }
    root = client.service(SERVICE)
    resp = client.post(root, 'edition_tools/launch', json=payload)
    if resp.status_code != 200:
        logging.error(
            'Error %i in %s : %s', resp.status_code, payload['tool'],
            resp.content
        )
        return None

    resp = resp.json()
    logging.debug('Tool %s done', payload['tool'])
    return resp


def merge(
    client: client_lib.ARTClient,
    rois: list[str],
):
    """Dilates a given list of rois with a given margin."""
    params = json.dumps({})
    payload = {'tool': 'fuion', 'rois': rois, 'params': params}
    root = client.service(SERVICE)
    resp = client.post(root, 'edition_tools/launch', json=payload)
    if resp.status_code != 200:
        logging.error(
            'Error %i in %s : %s', resp.status_code, payload['tool'],
            resp.content
        )
        return None

    resp = resp.json()
    logging.debug('Tool %s done', payload['tool'])
    return resp


def delete(
    client: client_lib.ARTClient,
    rois: list[str],
):
    """Dilates a given list of rois with a given margin."""
    params = json.dumps({})
    payload = {'tool': 'delete', 'rois': rois, 'params': params}
    root = client.service(SERVICE)
    resp = client.post(root, 'edition_tools/launch', json=payload)
    if resp.status_code != 200:
        logging.error(
            'Error %i in %s : %s', resp.status_code, payload['tool'],
            resp.content
        )
        return None

    resp = resp.json()
    logging.debug('Tool %s done', payload['tool'])
    return resp


def clone(
    client: client_lib.ARTClient,
    rois: list[str],
):
    """Dilates a given list of rois with a given margin."""
    params = json.dumps({})
    payload = {'tool': 'clone', 'rois': rois, 'params': params}
    root = client.service(SERVICE)
    resp = client.post(root, 'edition_tools/launch', json=payload)
    if resp.status_code != 200:
        logging.error(
            'Error %i in %s : %s', resp.status_code, payload['tool'],
            resp.content
        )
        return None

    resp = resp.json()
    logging.debug('Tool %s done', payload['tool'])
    return resp
